﻿using SafeCampus.Core.Options;
using TencentCloud.Common;
using TencentCloud.Common.Profile;
using TencentCloud.Sms.V20210111;
using TencentCloud.Sms.V20210111.Models;

namespace SafeCampus.Core.Utils.TXYSMS;

public  class TxySmsUtil: ISMSUtilService, IScoped
{
    private readonly SqlSugarScope _db;

    public TxySmsUtil()
    {
        _db = DbContext.DB;
    }

    /// <summary>
    /// 发送短信
    /// </summary>
    /// <param name="phone">new[] { $"+86{phone}" } 手机号码</param>
    /// <param name="content">new[] { code.ToString(), "5" } 内容参数</param>
    /// <returns></returns>
    public async Task<bool> SendSms(string[] phone, string[] content)
    {
        var setting = App.GetOptionsMonitor<TXSmsOptions>();
        Credential cred = new()
        {
            SecretId = setting.SecretId,
            SecretKey = setting.SecretKey
        };
        ClientProfile clientProfile = new();
        HttpProfile httpProfile = new()
        {
            Endpoint = ("sms.tencentcloudapi.com"),
        };
        clientProfile.HttpProfile = httpProfile;
        SmsClient client = new(cred, "ap-beijing", clientProfile);
        SendSmsRequest req = new()
        {
            PhoneNumberSet = phone,// new[] { $"+86{phone}" },
            SmsSdkAppId = setting.SmsAppId,
            SignName = setting.SmsSn,
            TemplateId = setting.VerificationCodeId,
            TemplateParamSet = content
        };
        SendSmsResponse resp = await client.SendSms(req);
        if (resp.SendStatusSet[0].Code == "Ok")
        {
            await CreateSMSLog(string.Join(",", phone), true, content==null?"":string.Join(",", content));
            return true;
        }
        await CreateSMSLog(string.Join(",",phone), false, content == null ? "" : string.Join(",", content));
        return false;
    }
    public async Task CreateSMSLog(string phone, bool sendState,string content)
    {
        string name = phone;//用户姓名
        string opAccount = "系统通知";//用户账号
        //日志表实体
        var devLogVisit = new SysLogVisit
        {
            Name = "短信通知",
            Category = CateGoryConst.CONFIG_SYS_ALARM_SMS,
            ExeStatus = sendState ? SysLogConst.SUCCESS : SysLogConst.FAIL,
            OpAddress = string.IsNullOrEmpty(content)? "泉江科技提醒您有新的预警信息，请您与预警平台-预警信息功能处查看详情" : content,
            OpIp = "",
            OpBrowser = "",
            OpOs = "",
            OpTime = DateTime.Now,
            OpUser = name,
            OpAccount = opAccount
        };
        await _db.CopyNew().InsertableWithAttr(devLogVisit).IgnoreColumns(true).SplitTable().ExecuteCommandAsync();//入库
    }
}