﻿using Mapster;
using MoYu.DependencyInjection;
using MoYu.FriendlyException;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json;
using SafeCampus.Application.Services.Business;
using SafeCampus.Application.Services.Business.CameraInfoService;
using SafeCampus.Application.Services.Business.PersonInfoService;
using SafeCampus.Application.Services.Business.Warn.Dto;
using SafeCampus.Application.Services.Business.Warn.Service;
using SafeCampus.Core.Utils;
using SafeCampus.Core.Utils.TXYSMS;
using System.Collections.Generic;

namespace SafeCampus.Web.Core.Controllers.Application.Business.Warn.Service;

public class WarnInfoService : DbRepository<WarnInfo>, IWarnInfoService, ITransient
{
    private readonly ISimpleCacheService _simpleCacheService;
    private readonly IConfigService _configService;
    private readonly ISMSUtilService _smsUtilService;
    private readonly ICameraInfoService _cameraInfoService;
    private readonly IPersonInfoService _personInfoService;
    private readonly ISysUserService _sysUserService;

    public WarnInfoService(ISimpleCacheService simpleCacheService, IConfigService configService, ISMSUtilService smsUtilService, ICameraInfoService cameraInfoService, IPersonInfoService personInfoService, ISysUserService sysUserService)
    {
        _simpleCacheService = simpleCacheService;
        _configService = configService;
        _smsUtilService = smsUtilService;
        _cameraInfoService = cameraInfoService;
        _personInfoService = personInfoService;
        _sysUserService = sysUserService;
    }

    public async Task<bool> Add(WarnInfoDto vm)
    {
        var model = vm.Adapt<WarnInfo>();
        await InsertAsync(model);
        var time = DateTime.Now;
        var count = 1;
        var pushState =
            (await _configService.GetByConfigKey(CateGoryConst.CONFIG_BIZ_DEFINE, BizConfigConst.BIZ_SMS_STATES))
            .ConfigValue.ToBoolean();
        var daySum =
            (await _configService.GetByConfigKey(CateGoryConst.CONFIG_BIZ_DEFINE, BizConfigConst.BIZ_SMS_STATES_DAY))
            .ConfigValue.ToInt();
       
        if (daySum > 1)
        {
            count = await Context.Queryable<WarnInfo>().CountAsync(x =>
                x.Tick.Year == time.Year && x.Tick.Month == time.Month && x.Tick.Day == time.Day);
        }

        if (!pushState || count < daySum) return true;
        {
            //var phone = await _cameraInfoService.GetInfoByCode(model.CameraId);
            var config = _simpleCacheService.Get<List<WarnGroupInfo>>(SafeCampusConst.WarnGroup);
            var scene = config.Where(x =>
                    x.Subset.Any(xx => xx.Code == model.AlarmType && xx.State && x.CameraId.Contains(model.CameraId))&&x.PushUserId.HasValue)
                .ToList();
            foreach (var warnGroupInfo in scene)
            {
               var user= await _sysUserService.Detail(new BaseIdInput { Id = warnGroupInfo.PushUserId.Value });
               if (user == null) continue;
               var issend = _simpleCacheService.Get<string>($"SMS_send{user.Phone}");
               if (issend != null) return true;
               var sendState = await _smsUtilService.SendSms(new[] { "+86" + user.Phone }, null);
               if (sendState)
               {
                   var configInterval = (await _configService.GetByConfigKey(CateGoryConst.CONFIG_BIZ_DEFINE,
                       BizConfigConst.BIZ_SMS_INTERVAL));
                   if (configInterval != null)
                   {
                       _simpleCacheService.Add($"SMS_send{user.Phone}", "发送成功", configInterval.ConfigValue.ToInt() * 60);
                   }
               }
            }
            //if (phone == null) return true;
            //场景名称
            //var sceneName = config.FirstOrDefault(x =>
            //    x.Subset.Any(xx => xx.Code == model.AlarmType) && x.CameraId.Contains(model.CameraId))?.Name;
            ////TODO 发送短信
            //var kzxx = "";
            //if (!string.IsNullOrEmpty(model.Extend))
            //{
            //    if (string.IsNullOrEmpty(model.PersonId))
            //    {
            //        var extend = JsonConvert.DeserializeObject<JObject>(model.Extend);
            //        kzxx = extend["age"] != null ? $"，年龄：{extend["age"]};年龄置信度：{Convert.ToInt32(Convert.ToDecimal(extend["ageProb"]) * 100)}%;" : "";
            //    }
            //    else
            //    {
            //        var pre = await _personInfoService.GetInfo(model.PersonId);
            //        if (pre!=null)
            //        {
            //            kzxx = $"，姓名：{pre.Name}，班级：{pre.PersonSetName}";
            //        }
            //    }
            //}
            //if (model.AlarmType == "crowd")
            //{
            //    kzxx = $"，人群聚集数量：{model.Count}";
            //}
            //var content = $"{model.Tick:yyyy-MM-dd HH:mm:ss}【{phone.SensorName}】触发【{model.AlarmTypeDesc}】预警，场景【{sceneName}】{kzxx}";

            //var issend = _simpleCacheService.Get<string>($"SMS_send{phone.SysUserItem.Phone}");
            //if (issend != null) return true;
            //var sendState = await _smsUtilService.SendSms(new[] { "+86" + Q3J5cHRvZ3JhbVV0aWw.U200RGVjcnlwdA(phone.SysUserItem.Phone) }, null);
            //if (sendState)
            //{
            //    var configInterval = (await _configService.GetByConfigKey(CateGoryConst.CONFIG_BIZ_DEFINE,
            //        BizConfigConst.BIZ_SMS_INTERVAL));
            //    if (configInterval != null)
            //    {
            //        _simpleCacheService.Add($"SMS_send{phone.SysUserItem.Phone}", "发送成功", configInterval.ConfigValue.ToInt() * 60);
            //    }

            //}

        }
        return true;
    }


    public async Task<bool> Delete(long Id)
    {
        var model = await GetFirstAsync(x => x.Id == Id);
        if (model == null)
        {
            throw Oops.Oh("信息不存在");
        }
        await DeleteAsync(model);
        return true;
    }

    public async Task<bool> Update(WarnInfoDto vm)
    {
        if (!vm.ID.HasValue)
        {
            throw Oops.Oh("请填写ID");
        }
        var model = await GetFirstAsync(p => p.Id == vm.ID);
        if (model == null)
        {
            throw Oops.Oh("信息不存在");
        }
        var res = vm.Adapt(model);
        await UpdateAsync(res);
        return true;
    }

    public async Task<WarnInfoDto> GetInfo(long Id)
    {
        var model = await Context.Queryable<WarnInfo>()
            .Includes(x => x.CameraInfoItem)
            .Includes(x => x.PersonInfoItem)
            .Includes(x => x.PersonSetInfoItem)
            .FirstAsync(x => x.Id == Id);
        if (model == null)
        {
            throw Oops.Oh("信息不存在！");
        }
        return model.Adapt<WarnInfoDto>();
    }

    public async Task<SqlSugarPagedList<WarnInfoList>> GetList(WarnInfoSearch search)
    {
        var config = _simpleCacheService.Get<List<WarnGroupInfo>>(SafeCampusConst.WarnGroup);
        var guolvIds = new List<long>();

        if (config != null)
        {
            var guolv = await Context.Queryable<WarnInfo>().Select(x => new { x.CameraId, x.AlarmType, x.Id }).ToListAsync();
            foreach (var warnGroupInfo in config)
            {
                if (warnGroupInfo.State)
                {
                    foreach (var warnGroupSubset in warnGroupInfo.Subset)
                    {
                        if (!warnGroupSubset.State)
                        {
                            guolvIds.AddRange(guolv
                                .Where(x => warnGroupSubset.CameraId.Contains(x.CameraId) &&
                                            x.AlarmType == warnGroupSubset.Code).Select(x => x.Id).ToList());
                        }
                    }
                }
                else
                {
                    guolvIds.AddRange(guolv
                        .Where(x => warnGroupInfo.CameraId.Contains(x.CameraId)).Select(x => x.Id).ToList());
                }
            }
        }

        var query = Context.Queryable<WarnInfo>()
            .Includes(x => x.CameraInfoItem)
            .Includes(x => x.PersonInfoItem)
            .Includes(x => x.PersonSetInfoItem, x => x.MajorInfoItem, x => x.DepartmentInfoItem)
            .WhereIF(search.WarnHand.HasValue, x => x.WarnHand == search.WarnHand)
            .WhereIF(!string.IsNullOrEmpty(search.AlarmType), x => x.AlarmType == search.AlarmType)
            .WhereIF(!string.IsNullOrEmpty(search.CameraId), x => x.CameraId == search.CameraId)
            .WhereIF(search.StartTick.HasValue, x => x.Tick >= search.StartTick)
            .WhereIF(search.EndTick.HasValue, x => x.Tick <= search.EndTick)
            .WhereIF(!string.IsNullOrEmpty(search.PersonId), x => x.PersonId == search.PersonId)
            .WhereIF(guolvIds.Any(), x => !guolvIds.Contains(x.Id));

        var list = await query.OrderByDescending(x => x.Tick)
            .ToPagedListAsyncMapster<WarnInfo, WarnInfoList>(search.PageNum, search.PageSize);
        return list;


    }

    public async Task<bool> HandWarn(HandWarnInput input)
    {
        var model = await GetFirstAsync(p => p.Id == input.Id);
        if (model == null)
        {
            throw Oops.Oh("信息不存在！");
        }
        model.Remark = input.Remark;
        model.WarnHand = 1;
        model.HandTime = DateTime.Now;
        await UpdateAsync(model);
        return true;
    }

    public async Task<List<WarnInfoList>> GetListNoPage(WarnInfoSearch search)
    {
        var config = _simpleCacheService.Get<List<WarnGroupInfo>>(SafeCampusConst.WarnGroup);
        var guolvIds = new List<long>();

        if (config != null)
        {
            var guolv = await Context.Queryable<WarnInfo>().Select(x => new { x.CameraId, x.AlarmType, x.Id }).ToListAsync();
            foreach (var warnGroupInfo in config)
            {
                if (warnGroupInfo.State)
                {
                    foreach (var warnGroupSubset in warnGroupInfo.Subset)
                    {
                        if (!warnGroupSubset.State)
                        {
                            guolvIds.AddRange(guolv
                                .Where(x => warnGroupSubset.CameraId.Contains(x.CameraId) &&
                                            x.AlarmType == warnGroupSubset.Code).Select(x => x.Id).ToList());
                        }
                    }
                }
                else
                {
                    guolvIds.AddRange(guolv
                        .Where(x => warnGroupInfo.CameraId.Contains(x.CameraId)).Select(x => x.Id).ToList());
                }
            }
        }
        var query = Context.Queryable<WarnInfo>()
            .Includes(x => x.CameraInfoItem, x => x.CameraGroupItem)
            .Includes(x => x.PersonInfoItem)
            .Includes(x => x.PersonSetInfoItem)
            .WhereIF(search.WarnHand.HasValue, x => x.WarnHand == search.WarnHand)
            .WhereIF(!string.IsNullOrEmpty(search.AlarmType), x => x.AlarmType == search.AlarmType)
            .WhereIF(search.AlarmTypes != null && search.AlarmTypes.Any(), x => search.AlarmTypes.Contains(x.AlarmType))
            .WhereIF(search.PersonSetIds != null && search.PersonSetIds.Any(), x => search.PersonSetIds.Contains(x.PersonSetId))
            .WhereIF(search.CameraIds != null && search.CameraIds.Any(), x => search.CameraIds.Contains(x.CameraId))
            .WhereIF(!string.IsNullOrEmpty(search.CameraId), x => x.CameraId == search.CameraId)
            .WhereIF(!string.IsNullOrEmpty(search.PersonId), x => x.PersonId == search.PersonId)
            .WhereIF(search.StartTick.HasValue, x => x.Tick >= search.StartTick)
            .WhereIF(search.EndTick.HasValue, x => x.Tick <= search.EndTick)
            .WhereIF(guolvIds.Any(), x => !guolvIds.Contains(x.Id));

        var list = await query.OrderByDescending(x => x.Tick)
            .ToListAsync();
        return list.Adapt<List<WarnInfoList>>();
    }
    public async Task<List<WarnType>> GetWarnType()
    {
        var config = _simpleCacheService.Get<List<WarnGroupInfo>>(SafeCampusConst.WarnGroup);
        var guolvIds = new List<long>();

        if (config != null)
        {
            var guolv = await Context.Queryable<WarnInfo>().Select(x => new { x.CameraId, x.AlarmType, x.Id }).ToListAsync();
            foreach (var warnGroupInfo in config)
            {
                if (warnGroupInfo.State)
                {
                    foreach (var warnGroupSubset in warnGroupInfo.Subset)
                    {
                        if (!warnGroupSubset.State)
                        {
                            guolvIds.AddRange(guolv
                                .Where(x => warnGroupSubset.CameraId.Contains(x.CameraId) &&
                                            x.AlarmType == warnGroupSubset.Code).Select(x => x.Id).ToList());
                        }
                    }
                }
                else
                {
                    guolvIds.AddRange(guolv
                        .Where(x => warnGroupInfo.CameraId.Contains(x.CameraId)).Select(x => x.Id).ToList());
                }
            }
        }
        var query = Context.Queryable<WarnInfo>()
            .WhereIF(guolvIds.Any(), x => !guolvIds.Contains(x.Id))
            .GroupBy(x => new { x.AlarmType, x.AlarmTypeDesc })
            .Select(xx => new WarnType
            {
                Name = xx.AlarmTypeDesc,
                Code = xx.AlarmType
            });
        return await query.ToListAsync();
    }

    public async Task<bool> DeleteBatch(List<long> Ids)
    {
        await Context.Deleteable<WarnInfo>().In(Ids).ExecuteCommandAsync();
        return true;
    }

    public async Task<int> GetWarnCount(WarnInfoSearch search)
    {
        var query = Context.Queryable<WarnInfo>()
            .Includes(x => x.CameraInfoItem, x => x.CameraGroupItem)
            .Includes(x => x.PersonInfoItem)
            .Includes(x => x.PersonSetInfoItem)
            .WhereIF(search.WarnHand.HasValue, x => x.WarnHand == search.WarnHand)
            .WhereIF(!string.IsNullOrEmpty(search.AlarmType), x => x.AlarmType == search.AlarmType)
            .WhereIF(search.AlarmTypes != null && search.AlarmTypes.Any(), x => search.AlarmTypes.Contains(x.AlarmType))
            .WhereIF(search.PersonSetIds != null && search.PersonSetIds.Any(),
                x => search.PersonSetIds.Contains(x.PersonSetId))
            .WhereIF(search.CameraIds != null && search.CameraIds.Any(), x => search.CameraIds.Contains(x.CameraId))
            .WhereIF(!string.IsNullOrEmpty(search.CameraId), x => x.CameraId == search.CameraId)
            .WhereIF(!string.IsNullOrEmpty(search.PersonId), x => x.PersonId == search.PersonId)
            .WhereIF(search.StartTick.HasValue, x => x.Tick >= search.StartTick)
            .WhereIF(search.EndTick.HasValue, x => x.Tick <= search.EndTick);

        var list = await query.CountAsync();
        return list;
    }
}