﻿using System.Management;
using System.Text;
using MoYu.FriendlyException;
using SafeCampus.Core.Utils;
using SafeCampus.Core.Utils.Machine;
using SharpCompress.Common;

namespace SafeCampus.Web.Core;
/// <summary>
/// 激活授权控制器
/// </summary>
[ApiDescriptionSettings(Tag = "激活授权控制器")]
[Route("sys/auth/activate")]
[AllowAnonymous]
public class ActivateAuthController : BaseController
{
    private readonly IMachineUtil _machineUtil;

    public ActivateAuthController(IMachineUtil machineUtil)
    {
        _machineUtil = machineUtil;
    }

    /// <summary>
    /// 获取机器码
    /// </summary>
    /// <param name="input"></param>
    /// <returns></returns>
    public async Task<dynamic> MachineCode()
    {
        string cpuId = await _machineUtil.GetProcessorId();
        string diskId = await _machineUtil.GetDiskId();
        string macAddress = await _machineUtil.GetMacAddress();
        var hardwareId = $"{cpuId}${diskId}${macAddress}";  // 可以用其他硬件信息替代
        var machineCode = Q3J5cHRvZ3JhbVV0aWw.U20yRW5jcnlwdA(Q3J5cHRvZ3JhbVV0aWw.U200RW5jcnlwdA(hardwareId));
        return machineCode;
    }
    /// <summary>
    /// 激活系统
    /// </summary>
    /// <param name="activationCode"></param>
    /// <returns></returns>
    public async Task<dynamic> Activation([FromBody]string activationCode)
    {
        var yuanwen =Q3J5cHRvZ3JhbVV0aWw.U20yRGVjcnlwdA(activationCode);
        var chaifen = yuanwen.Split('$');
        if (chaifen.Length!=4)
        {
            throw Oops.Oh("激活码错误！");
        }
        var isCorrect = DateTime.TryParse(chaifen[3],out var activeTime);
        if (!isCorrect)
        {
            throw Oops.Oh("激活码错误！");
        }
        if (chaifen[0] != await _machineUtil.GetProcessorId() || chaifen[1] != await _machineUtil.GetDiskId() || chaifen[2]!=await _machineUtil.GetMacAddress() || activeTime <= DateTime.Now)
        {
            throw Oops.Oh("激活码错误！");
        }
        var filePath = Path.Combine(App.HostEnvironment.ContentRootPath, "activation.config");
        await using StreamWriter writer = new StreamWriter(filePath);
        // 写入文本
        await writer.WriteLineAsync(Q3J5cHRvZ3JhbVV0aWw.U200RW5jcnlwdA(activationCode));
        return true;
    }
}